/*
  Copyright (c) 2013, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#ifndef _GMRF_MMSS_MMSS_H_
#define _GMRF_MMSS_MMSS_H_

#include <gmrf/gmrf.h>
#include <mmss/protocol.h>

#include "queue.h"


struct mmss {
	uint64_t now;
	mmss_queue_t packet_queue;
	mmss_queue_t scheduled_queue;
};

struct mmss_network {
	mmss_network_t *next;

	gmrf_iface_t *interfaces;

	size_t mtu;
};

struct mmss_packet {
	uint64_t sent;

	gmrf_iface_t *source;
	gmrf_iface_t *dest;

	size_t len;
	uint8_t data[];
};

struct mmss_scheduled {
	gmrf_t *node;
	gmrf_scheduled_func f;
	void *arg;
};


struct gmrf {
	gmrf_t *next;

	const char *name;

	mmss_t *mmss;
	gmrf_context_t *ctx;
	gmrf_iface_t *interfaces;

	unsigned rand_seed;

	const mmss_protocol_t *proto;
};

struct gmrf_iface {
	gmrf_iface_t *node_next;
	gmrf_iface_t *network_next;

	char *name;
	gmrf_addr_t address;

	gmrf_t *node;
	mmss_network_t *net;
};


const mmss_protocol_t* mmss_load_protocol(const char *module);

void mmss_add_iface(gmrf_t *node, mmss_network_t *net, const char *name, const gmrf_addr_t *address);

void mmss_dispatch(mmss_packet_t *packet);
void mmss_run_scheduled(mmss_scheduled_t *scheduled);


static inline int max(int a, int b) {
	return (a > b) ? a : b;
}

static inline int min(int a, int b) {
	return (a < b) ? a : b;
}


#endif /* _GMRF_MMSS_MMSS_H_ */
